import { Store } from "@willsoto/node-konfig-core";

let proxies: string[] = [];
let currentProxyIndex = 0;
let proxyData: string;
try {
    proxyData = Deno.readTextFileSync("proxies.txt");
    proxies = proxyData.split("\n").map((line) => line.trim()).filter((line) =>
        line.length > 0
    );
    console.log("[INFO] Num of proxies on the list:", proxies.length)
} catch (error) {
    console.error("[ERROR] Error reading proxy file:", error);
    console.log("[INFO] Proxies from a list of proxies will not be used");
}

export const getFetchClient = (konfigStore: Store): {
    (input: RequestInfo | URL, init?: RequestInit): Promise<Response>;
    (
        input: Request | URL | string,
        init?: RequestInit & {
            client: Deno.HttpClient;
        },
    ): Promise<Response>;
    (input: URL | Request | string, init?: RequestInit): Promise<Response>;
} => {
    if (
        Deno.env.get("PROXY") || konfigStore.get("networking.proxy") ||
        (proxies.length > 0)
    ) {
        return async (
            input: RequestInfo | URL,
            init?: RequestInit,
        ) => {
            const proxyUrl = Deno.env.get("PROXY") ||
                konfigStore.get("networking.proxy") as string ||
                (proxies.length > 0 ? proxies[currentProxyIndex] : null);
            if (!proxyUrl) {
                throw new Error("No proxy available");
            }
            const client = Deno.createHttpClient({
                proxy: {
                    url: proxyUrl,
                },
            });

            currentProxyIndex = (currentProxyIndex + 1) % proxies.length;

            const fetchRes = await fetch(input, {
                client,
                headers: init?.headers,
                method: init?.method,
                body: init?.body,
            });

            return new Response(fetchRes.body, {
                status: fetchRes.status,
                headers: fetchRes.headers,
            });
        };
    }

    return globalThis.fetch;
};
